﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {

  public abstract class StatefullExpression<T> : Expression {

    public readonly T State;

    private int hashCode;
    private int HashCode {
      get {
        if (hashCode == default(int)) hashCode = CalcHashCode();
        return hashCode;
      }
    }

    protected StatefullExpression(T state) {
      this.State = state;
    }

    protected virtual int CalcHashCode() {
      return this.State.GetHashCode();
    }

    public override bool Equals(object obj) {
      if (ReferenceEquals(this, obj))
        return true;

      if (this.GetType() != obj.GetType())
        return false;

      var other = (StatefullExpression<T>)obj;

      return ReferenceEquals(this.State, other.State) ||
             this.GetHashCode() == other.GetHashCode();
    }

    public override int GetHashCode() {
      return HashCode;
    }
  }
}